/***********************************************************************//**
 * @file		main.c
 * @brief		Simple main function for PDCP for AVR testing @n
 * 				Main function for testing proper HAL operation
 * @details		This main sends amount of messages (SET in MSG_AMOUNT_TEST) 
 *				from ARBITRATOR to DEVICE. If messages were transmitted without
 *				lossess, HAL works according to assumptions.
 *				Project contains not optimized and completed HLL layer files, 
 *				therefore only HAL is tested. 
 *
 * @attention	Code was optimized to NIMRON board!
 *				
 *
 * @author		Andrzej Zamojski ( NTNU )
 *
 * @date		10 VI 2012
 **************************************************************************/


/* Includes ======================================== */
#include "config.h"
#include <stdio.h> 		//sprintf

#include "uCmaskFile.h"
#include "MCP2515control.h"
#include "functions.h"
#include "irqHandlers.h"



/* Extern flags of CAN controller registers...
 		     ...indispensible for testing ========== */
extern volatile uint8_t rxBuf0Full;
extern volatile uint8_t rxBuf1Full;
extern volatile uint8_t txBuf0Ready;
extern volatile uint8_t txBuf1Ready;
extern volatile uint8_t txBuf2Ready;

volatile uint16_t time;						//* timer counter
volatile uint8_t flag;						//* flag of time start


/* Main ======================================== */
int main(){

	initLowLevelProtocolHardware();			//* initialization of HAL (user has to be aware of adjusting timer...
											//* ... counter to provide good time measurement!
	sei();

	printUsart("PROGRAM START\r");			//* print to terminal

#if ( CONFIG_TESTING_HAL )
	time = 0;								//* timer counter -> reset
	flag = 0;								//* time measurement start flag -> reset
	volatile uint8_t sendOnce = 1;			//* help in sending first message
	volatile uint8_t arbStop = 0;			//* stop messages sending for ARBITRATOR

	volatile uint32_t arbCnt = 0;			//* arbitrator sent message counter
	volatile uint32_t devCnt = 0;			//* device received message counter

	struct can_msg msgToReceiving;			//* structure for receiving
	struct can_msg msgToSending;			//* structure for sending
	

	msgToSending.id = 0xFF;					//* id of recipient
	msgToSending.len = 0x6;					//* message length
	for (int i =0; i < msgToSending.len; i++)
		msgToSending.data[i] = DATA_TO_SEND_1;	//* filling structure with data DATA_TO_SEND_1 (config.h)
	
	
	TIMER0_TCNT = 0;						//* TIMER0 RESET
#endif


#if ( CONFIG_BUS_MODE == BUS_DEVICE )							//* DEVICE code

	#if ( CONFIG_TESTING_HAL )
	while(1){

		if ( rxBuf0Full ) {										//* if message received by CAN controller
			flag = 1;											//* start time counting
			receiveCanMsg ( &msgToReceiving, REC_REG_0 );		//* receive msg
			if ( msgToReceiving.data[0] == DATA_TO_SEND_1 )		//* check correctness
				devCnt++;										//* increment counter
		}
		if ( rxBuf1Full ) {										//* as above ----- || ------
			flag = 1;	// start time counting
			receiveCanMsg ( &msgToReceiving, REC_REG_1 );
			if ( msgToReceiving.data[0] == DATA_TO_SEND_1 )
				devCnt++;
		}
		
		if ( devCnt == MSG_AMOUNT_TEST ) {						//* if amount of received messages is correct...
			volatile unsigned char buf[40];						//* print to terminal and toggle diode
				printUsart("This is test of HAL capacity\r");
				printUsart("This message is displayed only in case of... \r");
				printUsart("... lossless transmission.\r");
			sprintf(buf,"It was received %d msg\r",MSG_AMOUNT_TEST);
				printUsart(buf);
			sprintf(buf,"During: %d.%d s.\r",(time / 10),(time % 10));
				printUsart(buf);

			HW_PORT_LED ^= ( 1 << HW_PORT_LED0 );
			devCnt = 0;											//* receiving message counter -> reset
		}
	}
	#endif





#elif ( CONFIG_BUS_MODE == BUS_ARBITRATOR )						//* ARBITRATOR code
	while(1){
		
	#if ( CONFIG_TESTING_HAL )	
	
	//* if transmitting buffer ready and amount of messages smaller than MSG_AMOUNT_TEST, trigger sending	
	if ( (txBuf0Ready || sendOnce) && 
		( arbCnt < MSG_AMOUNT_TEST) && !arbStop ){	
		sendCanMsg( &msgToSending, SEND_REG_0 );				//* send the message
		arbCnt++;												//* increment arbitrator counter
		sendOnce = 0;											//* base on interrupts from CAN controller		
	}
	
	/* as above */
	if ( (txBuf1Ready || sendOnce) && 
		( arbCnt < MSG_AMOUNT_TEST) && !arbStop ){
		sendCanMsg( &msgToSending, SEND_REG_1);					//* send the message
		arbCnt++;												//* increment arbitrator counter
		sendOnce = 0;
	}

	/* as above */
	if ( (txBuf2Ready || sendOnce) && 
		( arbCnt < MSG_AMOUNT_TEST) && !arbStop ){
		sendCanMsg( &msgToSending, SEND_REG_2);					//* send the message
		arbCnt++;												//* increment arbitrator counter							
		sendOnce = 0;
	}


	//* if number of sent messages equals MSG_AMOUNT_TEST, stop sending, print to terminal and toggle diode
	if ( (arbCnt == MSG_AMOUNT_TEST) && !arbStop ){
		volatile char tab[40];
		sprintf(tab,"\r%d of msgs were sent by ARB\r",MSG_AMOUNT_TEST);
		printUsart(tab);
		HW_PORT_LED ^= ( 1 << HW_PORT_LED0 );					//* toggle diode
		arbStop = 1;											//* stop sending
	}
	#endif		
	}

#endif

	return 0;
}

